@@ -21,10 +21,12 @@ var ErrNoThreads = errors.New("no threads found in core file")
 // 0x0000000000400000  0x000000000044f000  0x0000000000000000
 // but then it's partially overwritten with an RW mapping whose data is stored
 // in the core file:
-// Type           Offset             VirtAddr           PhysAddr
-//                FileSiz            MemSiz              Flags  Align
-// LOAD           0x0000000000004000 0x000000000049a000 0x0000000000000000
-//                0x0000000000002000 0x0000000000002000  RW     1000
+//
+//	Type           Offset             VirtAddr           PhysAddr
+//	               FileSiz            MemSiz              Flags  Align
+//	LOAD           0x0000000000004000 0x000000000049a000 0x0000000000000000
+//	               0x0000000000002000 0x0000000000002000  RW     1000
+//
 // This can be represented in a SplicedMemory by adding the original region,
 // then putting the RW mapping on top of it.
 type splicedMemory struct {
@@ -248,7 +250,9 @@ func (p *process) WriteBreakpoint(*proc.Breakpoint) error {
 func (p *process) Recorded() (bool, string) { return true, "" }
 
 // Restart will only return an error for core files, as they are not executing.
-func (p *process) Restart(string) (proc.Thread, error) { return nil, ErrContinueCore }
+func (p *process) Restart(*proc.ContinueOnceContext, string) (proc.Thread, error) {
+	return nil, ErrContinueCore
+}
 
 // ChangeDirection will only return an error as you cannot continue a core process.
 func (p *process) ChangeDirection(proc.Direction) error { return ErrContinueCore }
@@ -415,7 +419,7 @@ func (p *process) ClearInternalBreakpoints() error {
 
 // ContinueOnce will always return an error because you
 // cannot control execution of a core file.
-func (p *process) ContinueOnce() (proc.Thread, proc.StopReason, error) {
+func (p *process) ContinueOnce(cctx *proc.ContinueOnceContext) (proc.Thread, proc.StopReason, error) {
 	return nil, proc.StopUnknown, ErrContinueCore
 }
 
@@ -427,7 +431,7 @@ func (p *process) StepInstruction() error {
 
 // RequestManualStop will return nil and have no effect
 // as you cannot control execution of a core file.
-func (p *process) RequestManualStop() error {
+func (p *process) RequestManualStop(cctx *proc.ContinueOnceContext) error {
 	return nil
 }
 
