@@ -338,17 +338,17 @@ func afterLastArgAddr(vars []*Variable) uint64 {
 }
 
 // setValue writes the value of srcv to dstv.
-// * If srcv is a numerical literal constant and srcv is of a compatible type
-//   the necessary type conversion is performed.
-// * If srcv is nil and dstv is of a nil'able type then dstv is nilled.
-// * If srcv is the empty string and dstv is a string then dstv is set to the
-//   empty string.
-// * If dstv is an "interface {}" and srcv is either an interface (possibly
-//   non-empty) or a pointer shaped type (map, channel, pointer or struct
-//   containing a single pointer field) the type conversion to "interface {}"
-//   is performed.
-// * If srcv and dstv have the same type and are both addressable then the
-//   contents of srcv are copied byte-by-byte into dstv
+//   - If srcv is a numerical literal constant and srcv is of a compatible type
+//     the necessary type conversion is performed.
+//   - If srcv is nil and dstv is of a nil'able type then dstv is nilled.
+//   - If srcv is the empty string and dstv is a string then dstv is set to the
+//     empty string.
+//   - If dstv is an "interface {}" and srcv is either an interface (possibly
+//     non-empty) or a pointer shaped type (map, channel, pointer or struct
+//     containing a single pointer field) the type conversion to "interface {}"
+//     is performed.
+//   - If srcv and dstv have the same type and are both addressable then the
+//     contents of srcv are copied byte-by-byte into dstv
 func (scope *EvalScope) setValue(dstv, srcv *Variable, srcExpr string) error {
 	srcv.loadValue(loadSingleValue)
 
@@ -2038,10 +2038,18 @@ func (v *Variable) mapAccess(idx *Variable) (*Variable, error) {
 		return nil, fmt.Errorf("can not access unreadable map: %v", v.Unreadable)
 	}
 
+	lcfg := loadFullValue
+	if idx.Kind == reflect.String && int64(len(constant.StringVal(idx.Value))) == idx.Len && idx.Len > int64(lcfg.MaxStringLen) {
+		// If the index is a string load as much of the keys to at least match the length of the index.
+		//TODO(aarzilli): when struct literals are implemented this needs to be
+		//done recursively for literal struct fields.
+		lcfg.MaxStringLen = int(idx.Len)
+	}
+
 	first := true
 	for it.next() {
 		key := it.key()
-		key.loadValue(loadFullValue)
+		key.loadValue(lcfg)
 		if key.Unreadable != nil {
 			return nil, fmt.Errorf("can not access unreadable map: %v", key.Unreadable)
 		}
@@ -2095,7 +2103,7 @@ func (v *Variable) reslice(low int64, high int64) (*Variable, error) {
 	wrong := false
 	cptrNeedsFakeSlice := false
 	if v.Flags&VariableCPtr == 0 {
-		wrong = low < 0 || low >= v.Len || high < 0 || high > v.Len
+		wrong = low < 0 || low > v.Len || high < 0 || high > v.Len
 	} else {
 		wrong = low < 0 || high < 0
 		if high == 0 {
