@@ -83,6 +83,8 @@ type Target struct {
 	// can be given a unique address.
 	fakeMemoryRegistry    []*compositeMemory
 	fakeMemoryRegistryMap map[string]*compositeMemory
+
+	cctx *ContinueOnceContext
 }
 
 type KeepSteppingBreakpoints uint8
@@ -198,6 +200,7 @@ func NewTarget(p ProcessInternal, pid int, currentThread Thread, cfg NewTargetCo
 		currentThread: currentThread,
 		CanDump:       cfg.CanDump,
 		pid:           pid,
+		cctx:          &ContinueOnceContext{},
 	}
 
 	if recman, ok := p.(RecordingManipulationInternal); ok {
@@ -265,7 +268,7 @@ func (t *Target) Valid() (bool, error) {
 // Currently only non-recorded processes running on AMD64 support
 // function calls.
 func (t *Target) SupportsFunctionCalls() bool {
-	return t.Process.BinInfo().Arch.Name == "amd64"
+	return t.Process.BinInfo().Arch.Name == "amd64" || t.Process.BinInfo().Arch.Name == "arm64"
 }
 
 // ClearCaches clears internal caches that should not survive a restart.
@@ -283,7 +286,7 @@ func (t *Target) ClearCaches() {
 // Restarting of a normal process happens at a higher level (debugger.Restart).
 func (t *Target) Restart(from string) error {
 	t.ClearCaches()
-	currentThread, err := t.recman.Restart(from)
+	currentThread, err := t.recman.Restart(t.cctx, from)
 	if err != nil {
 		return err
 	}
@@ -395,8 +398,8 @@ func (t *Target) createUnrecoveredPanicBreakpoint() {
 	if err == nil {
 		bp, err := t.SetBreakpoint(unrecoveredPanicID, panicpcs[0], UserBreakpoint, nil)
 		if err == nil {
-			bp.Name = UnrecoveredPanic
-			bp.Variables = []string{"runtime.curg._panic.arg"}
+			bp.Logical.Name = UnrecoveredPanic
+			bp.Logical.Variables = []string{"runtime.curg._panic.arg"}
 		}
 	}
 }
@@ -407,7 +410,14 @@ func (t *Target) createFatalThrowBreakpoint() {
 	if err == nil {
 		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
 		if err == nil {
-			bp.Name = FatalThrow
+			bp.Logical.Name = FatalThrow
+		}
+	}
+	fatalpcs, err = FindFunctionLocation(t.Process, "runtime.fatal", 0)
+	if err == nil {
+		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
+		if err == nil {
+			bp.Logical.Name = FatalThrow
 		}
 	}
 }
@@ -464,9 +474,23 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 	return results
 }
 
+// ResumeNotify specifies a channel that will be closed the next time
+// Continue finishes resuming the target.
+func (t *Target) ResumeNotify(ch chan<- struct{}) {
+	t.cctx.ResumeChan = ch
+}
+
+// RequestManualStop attempts to stop all the process' threads.
+func (t *Target) RequestManualStop() error {
+	t.cctx.StopMu.Lock()
+	defer t.cctx.StopMu.Unlock()
+	t.cctx.manualStopRequested = true
+	return t.proc.RequestManualStop(t.cctx)
+}
+
 const (
 	FakeAddressBase     = 0xbeef000000000000
-	fakeAddressUnresolv = 0xbeed000000000000 // this address never resloves to memory
+	fakeAddressUnresolv = 0xbeed000000000000 // this address never resolves to memory
 )
 
 // newCompositeMemory creates a new compositeMemory object and registers it.
@@ -593,4 +617,6 @@ func (*dummyRecordingManipulation) ClearCheckpoint(int) error { return ErrNotRec
 
 // Restart will always return an error in the native proc backend, only for
 // recorded traces.
-func (*dummyRecordingManipulation) Restart(string) (Thread, error) { return nil, ErrNotRecorded }
+func (*dummyRecordingManipulation) Restart(*ContinueOnceContext, string) (Thread, error) {
+	return nil, ErrNotRecorded
+}
