@@ -78,6 +78,8 @@ const (
 	BuildModePlugin
 	BuildModeExternalLinker
 	AllNonOptimized
+	// LinkDisableDWARF enables '-ldflags="-w"'.
+	LinkDisableDWARF
 )
 
 // BuildFixture will compile the fixture 'name' using the provided build flags.
@@ -114,9 +116,14 @@ func BuildFixture(name string, flags BuildFlags) Fixture {
 		// Work-around for https://github.com/golang/go/issues/13154
 		buildFlags = append(buildFlags, "-ldflags=-linkmode internal")
 	}
+	ldflagsv := []string{}
 	if flags&LinkStrip != 0 {
-		buildFlags = append(buildFlags, "-ldflags=-s")
+		ldflagsv = append(ldflagsv, "-s")
 	}
+	if flags&LinkDisableDWARF != 0 {
+		ldflagsv = append(ldflagsv, "-w")
+	}
+	buildFlags = append(buildFlags, "-ldflags="+strings.Join(ldflagsv, " "))
 	gcflagsv := []string{}
 	if flags&EnableInlining == 0 {
 		gcflagsv = append(gcflagsv, "-l")
@@ -256,20 +263,20 @@ func AllowRecording(t testing.TB) {
 // MustHaveRecordingAllowed skips this test if recording is not allowed
 //
 // Not all the tests can be run with a recording:
-// - some fixtures never terminate independently (loopprog,
-//   testnextnethttp) and can not be recorded
-// - some tests assume they can interact with the target process (for
-//   example TestIssue419, or anything changing the value of a variable),
-//   which we can't do on with a recording
-// - some tests assume that the Pid returned by the process is valid, but
-//   it won't be at replay time
-// - some tests will start the fixture but not never execute a single
-//   instruction, for some reason rr doesn't like this and will print an
-//   error if it happens
-// - many tests will assume that we can return from a runtime.Breakpoint,
-//   with a recording this is not possible because when the fixture ran it
-//   wasn't attached to a debugger and in those circumstances a
-//   runtime.Breakpoint leads directly to a crash
+//   - some fixtures never terminate independently (loopprog,
+//     testnextnethttp) and can not be recorded
+//   - some tests assume they can interact with the target process (for
+//     example TestIssue419, or anything changing the value of a variable),
+//     which we can't do on with a recording
+//   - some tests assume that the Pid returned by the process is valid, but
+//     it won't be at replay time
+//   - some tests will start the fixture but not never execute a single
+//     instruction, for some reason rr doesn't like this and will print an
+//     error if it happens
+//   - many tests will assume that we can return from a runtime.Breakpoint,
+//     with a recording this is not possible because when the fixture ran it
+//     wasn't attached to a debugger and in those circumstances a
+//     runtime.Breakpoint leads directly to a crash
 //
 // Some of the tests using runtime.Breakpoint (anything involving variable
 // evaluation and TestWorkDir) have been adapted to work with a recording.
@@ -314,16 +321,21 @@ func MustSupportFunctionCalls(t *testing.T, testBackend string) {
 		t.Skip("this version of Go does not support function calls")
 	}
 
-	if testBackend == "rr" || (runtime.GOOS == "darwin" && testBackend == "native") {
+	if runtime.GOOS == "darwin" && testBackend == "native" {
 		t.Skip("this backend does not support function calls")
 	}
 
-	if runtime.GOOS == "darwin" && os.Getenv("TRAVIS") == "true" {
+	if runtime.GOOS == "darwin" && os.Getenv("TRAVIS") == "true" && runtime.GOARCH == "amd64" {
 		t.Skip("function call injection tests are failing on macOS on Travis-CI (see #1802)")
 	}
-	if runtime.GOARCH == "arm64" || runtime.GOARCH == "386" {
+	if runtime.GOARCH == "386" {
 		t.Skip(fmt.Errorf("%s does not support FunctionCall for now", runtime.GOARCH))
 	}
+	if runtime.GOARCH == "arm64" {
+		if !goversion.VersionAfterOrEqual(runtime.Version(), 1, 19) {
+			t.Skip("this version of Go does not support function calls")
+		}
+	}
 }
 
 // DefaultTestBackend changes the value of testBackend to be the default
@@ -379,3 +391,15 @@ func MustHaveCgo(t *testing.T) {
 		t.Skip("Cgo not enabled")
 	}
 }
+
+func RegabiSupported() bool {
+	// Tracks regabiSupported variable in ParseGOEXPERIMENT internal/buildcfg/exp.go
+	switch {
+	case goversion.VersionAfterOrEqual(runtime.Version(), 1, 18):
+		return runtime.GOARCH == "amd64" || runtime.GOARCH == "arm64" || runtime.GOARCH == "ppc64le" || runtime.GOARCH == "ppc64"
+	case goversion.VersionAfterOrEqual(runtime.Version(), 1, 17):
+		return runtime.GOARCH == "amd64" && (runtime.GOOS == "android" || runtime.GOOS == "linux" || runtime.GOOS == "darwin" || runtime.GOOS == "windows")
+	default:
+		return false
+	}
+}
