@@ -136,6 +136,18 @@ func (s *RPCServer) Command(command api.DebuggerCommand, cb service.RPCCallback)
 	cb.Return(out, nil)
 }
 
+type GetBufferedTracepointsIn struct {
+}
+
+type GetBufferedTracepointsOut struct {
+	TracepointResults []api.TracepointResult
+}
+
+func (s *RPCServer) GetBufferedTracepoints(arg GetBufferedTracepointsIn, out *GetBufferedTracepointsOut) error {
+	out.TracepointResults = s.debugger.GetBufferedTracepoints()
+	return nil
+}
+
 type GetBreakpointIn struct {
 	Id   int
 	Name string
@@ -215,6 +227,7 @@ func (s *RPCServer) Ancestors(arg AncestorsIn, out *AncestorsOut) error {
 }
 
 type ListBreakpointsIn struct {
+	All bool
 }
 
 type ListBreakpointsOut struct {
@@ -223,7 +236,7 @@ type ListBreakpointsOut struct {
 
 // ListBreakpoints gets all breakpoints.
 func (s *RPCServer) ListBreakpoints(arg ListBreakpointsIn, out *ListBreakpointsOut) error {
-	out.Breakpoints = s.debugger.Breakpoints()
+	out.Breakpoints = s.debugger.Breakpoints(arg.All)
 	return nil
 }
 
@@ -251,6 +264,18 @@ func (s *RPCServer) CreateBreakpoint(arg CreateBreakpointIn, out *CreateBreakpoi
 	return nil
 }
 
+type CreateEBPFTracepointIn struct {
+	FunctionName string
+}
+
+type CreateEBPFTracepointOut struct {
+	Breakpoint api.Breakpoint
+}
+
+func (s *RPCServer) CreateEBPFTracepoint(arg CreateEBPFTracepointIn, out *CreateEBPFTracepointOut) error {
+	return s.debugger.CreateEBPFTracepoint(arg.FunctionName)
+}
+
 type ClearBreakpointIn struct {
 	Id   int
 	Name string
@@ -362,7 +387,7 @@ func (s *RPCServer) ListThreads(arg ListThreadsIn, out *ListThreadsOut) (err err
 	}
 	s.debugger.LockTarget()
 	defer s.debugger.UnlockTarget()
-	out.Threads = api.ConvertThreads(threads)
+	out.Threads = api.ConvertThreads(threads, s.debugger.ConvertThreadBreakpoint)
 	return nil
 }
 
@@ -385,7 +410,7 @@ func (s *RPCServer) GetThread(arg GetThreadIn, out *GetThreadOut) error {
 	}
 	s.debugger.LockTarget()
 	defer s.debugger.UnlockTarget()
-	out.Thread = api.ConvertThread(t)
+	out.Thread = api.ConvertThread(t, s.debugger.ConvertThreadBreakpoint(t))
 	return nil
 }
 
@@ -496,7 +521,7 @@ type EvalOut struct {
 	Variable *api.Variable
 }
 
-// EvalVariable returns a variable in the specified context.
+// Eval returns a variable in the specified context.
 //
 // See https://github.com/go-delve/delve/blob/master/Documentation/cli/expr.md
 // for a description of acceptable values of arg.Expr.
@@ -607,15 +632,21 @@ type ListGoroutinesOut struct {
 // If arg.Filters are specified the list of returned goroutines is filtered
 // applying the specified filters.
 // For example:
-//    ListGoroutinesFilter{ Kind: ListGoroutinesFilterUserLoc, Negated: false, Arg: "afile.go" }
+//
+//	ListGoroutinesFilter{ Kind: ListGoroutinesFilterUserLoc, Negated: false, Arg: "afile.go" }
+//
 // will only return goroutines whose UserLoc contains "afile.go" as a substring.
 // More specifically a goroutine matches a location filter if the specified
 // location, formatted like this:
-//    filename:lineno in function
+//
+//	filename:lineno in function
+//
 // contains Arg[0] as a substring.
 //
 // Filters can also be applied to goroutine labels:
-//    ListGoroutineFilter{ Kind: ListGoroutinesFilterLabel, Negated: false, Arg: "key=value" }
+//
+//	ListGoroutineFilter{ Kind: ListGoroutinesFilterLabel, Negated: false, Arg: "key=value" }
+//
 // this filter will only return goroutines that have a key=value label.
 //
 // If arg.GroupBy is not GoroutineFieldNone then the goroutines will
@@ -675,15 +706,15 @@ type FindLocationOut struct {
 
 // FindLocation returns concrete location information described by a location expression.
 //
-//  loc ::= <filename>:<line> | <function>[:<line>] | /<regex>/ | (+|-)<offset> | <line> | *<address>
-//  * <filename> can be the full path of a file or just a suffix
-//  * <function> ::= <package>.<receiver type>.<name> | <package>.(*<receiver type>).<name> | <receiver type>.<name> | <package>.<name> | (*<receiver type>).<name> | <name>
-//  * <function> must be unambiguous
-//  * /<regex>/ will return a location for each function matched by regex
-//  * +<offset> returns a location for the line that is <offset> lines after the current line
-//  * -<offset> returns a location for the line that is <offset> lines before the current line
-//  * <line> returns a location for a line in the current file
-//  * *<address> returns the location corresponding to the specified address
+//	loc ::= <filename>:<line> | <function>[:<line>] | /<regex>/ | (+|-)<offset> | <line> | *<address>
+//	* <filename> can be the full path of a file or just a suffix
+//	* <function> ::= <package>.<receiver type>.<name> | <package>.(*<receiver type>).<name> | <receiver type>.<name> | <package>.<name> | (*<receiver type>).<name> | <name>
+//	  <function> must be unambiguous
+//	* /<regex>/ will return a location for each function matched by regex
+//	* +<offset> returns a location for the line that is <offset> lines after the current line
+//	* -<offset> returns a location for the line that is <offset> lines before the current line
+//	* <line> returns a location for a line in the current file
+//	* *<address> returns the location corresponding to the specified address
 //
 // NOTE: this function does not actually set breakpoints.
 func (c *RPCServer) FindLocation(arg FindLocationIn, out *FindLocationOut) error {
@@ -986,3 +1017,15 @@ func (s *RPCServer) CreateWatchpoint(arg CreateWatchpointIn, out *CreateWatchpoi
 	out.Breakpoint, err = s.debugger.CreateWatchpoint(arg.Scope.GoroutineID, arg.Scope.Frame, arg.Scope.DeferredCall, arg.Expr, arg.Type)
 	return err
 }
+
+type BuildIDIn struct {
+}
+
+type BuildIDOut struct {
+	BuildID string
+}
+
+func (s *RPCServer) BuildID(arg BuildIDIn, out *BuildIDOut) error {
+	out.BuildID = s.debugger.BuildID()
+	return nil
+}
